import java.util.*;

/**
 * The class Timer implements a timer.
 * @author  Stephan Fischli
 * @version 1.0
 */
public class Timer extends Observable implements Runnable {


    /**
     * The number of ticks.
     */
    private int ticks;

    /**
     * The time interval (in milliseconds) of a tick.
     */
    private int interval;

    /**
     * The thread which triggers the ticks.
     */
    private Thread thread;

    /**
     * Creates a Timer object.
     * @param interval the time interval (in milliseconds) of the timer
     */
    public Timer( int interval ) {
        this.interval = interval;
    }

    /**
     * Returns the time (in seconds) of the timer.
     */
    public double getTime() {
        return ticks*interval/1000.0;
    }


    /**
     * Returns true if the timer is running, otherwise false.
     */
    public boolean isRunning() {
        return thread != null;
    }

    /**
     * Starts the timer.
     */
    public void start() {
      if ( thread == null ) {
          thread = new Thread( this );
          thread.setPriority( Thread.MAX_PRIORITY );
          thread.start();
      }
    }

    /**
     * Stops the timer.
     */
    public void stop() {
      if ( thread != null ) {
          thread = null;
      }
    }

    /**
     * Resets the time of the timer.
     */
    public void reset() {
      ticks = 0;
    }

    /**
     * Tiggers ticks at the expiration of the time interval.
     */
    public void run() {
      while ( isRunning() ) {
        try { Thread.sleep( interval ); }
        catch ( InterruptedException e ) {}
        if( isRunning() ) {
          ticks++;
        }
      }
    }
}

